// Krisztian Pinter 2014

#include "GUT.h"

static int              GUT_image_width;
static int              GUT_image_height;
static unsigned char    GUT_drawing_color[3];
static unsigned char *  GUT_image_data;

void GUT_init( int w, int h )
{
    GUT_image_width  = w;
    GUT_image_height = h;
    GUT_image_data = malloc(3 * GUT_image_width * GUT_image_height);
    memset(GUT_image_data, 255, 3 * GUT_image_width * GUT_image_height);
}

int GUT_is_init()
{
    return GUT_image_data != 0;
}

void GUT_close()
{
    free(GUT_image_data);
}

void GUT_output_bmp(char * fname)
{
    FILE *f;
    int filesize = 54 + 3 * GUT_image_width * GUT_image_height;

    unsigned char bmpfileheader[14] = {'B','M', 0,0,0,0, 0,0, 0,0, 54,0,0,0};
    unsigned char bmpinfoheader[40] = {40,0,0,0, 0,0,0,0, 0,0,0,0, 1,0, 24,0};
    unsigned char bmppad[3] = {0,0,0};

    bmpfileheader[ 2] = (unsigned char)(filesize    );
    bmpfileheader[ 3] = (unsigned char)(filesize>> 8);
    bmpfileheader[ 4] = (unsigned char)(filesize>>16);
    bmpfileheader[ 5] = (unsigned char)(filesize>>24);

    bmpinfoheader[ 4] = (unsigned char)(GUT_image_width     );
    bmpinfoheader[ 5] = (unsigned char)(GUT_image_width >> 8);
    bmpinfoheader[ 6] = (unsigned char)(GUT_image_width >>16);
    bmpinfoheader[ 7] = (unsigned char)(GUT_image_width >>24);
    bmpinfoheader[ 8] = (unsigned char)(GUT_image_height    );
    bmpinfoheader[ 9] = (unsigned char)(GUT_image_height>> 8);
    bmpinfoheader[10] = (unsigned char)(GUT_image_height>>16);
    bmpinfoheader[11] = (unsigned char)(GUT_image_height>>24);

    f = fopen(fname,"wb");
    fwrite(bmpfileheader,1,14,f);
    fwrite(bmpinfoheader,1,40,f);
    for(int i = 0; i < GUT_image_height; ++i)
    {
        fwrite(GUT_image_data + (GUT_image_width*(GUT_image_height-i-1)*3),
            3, GUT_image_width, f);
        fwrite(bmppad, 1, (4-(GUT_image_width*3)%4)%4, f);
    }
    fclose(f);
}

inline void GUT_set_color( int r, int g, int b )
{
    GUT_drawing_color[2] = r;
    GUT_drawing_color[1] = g;
    GUT_drawing_color[0] = b;
}

inline void GUT_draw_pixel( int x, int y )
{
    if( 0 <= x && x < GUT_image_width &&
        0 <= y && y < GUT_image_height )
        memcpy( GUT_image_data + ( y * GUT_image_width + x ) * 3,
        GUT_drawing_color, 3 );
}

void GUT_draw_line( int x0, int y0, int x1, int y1 )
{
    int dx = abs(x1-x0);
    int dy = abs(y1-y0);
    int sx = (x0 < x1)? 1 : -1;
    int sy = (y0 < y1)? 1 : -1;
    int err = dx-dy;

    int e2;
    while(1)
    {
        GUT_draw_pixel( x0, y0 );
        if( x0 == x1 && y0 == y1 ) break;
        e2 = err << 1;
        if( e2 > -dy ) err -= dy, x0 += sx;
        if( e2 <  dx ) err += dx, y0 += sy;
    }
}

void GUT_draw_circle( int x0, int y0, int rad )
{
    int x = rad, y = 0;
    int rad_err = 1 - x;
 
    while(x >= y)
    {
        GUT_draw_pixel(  x + x0,  y + y0 );
        GUT_draw_pixel(  y + x0,  x + y0 );
        GUT_draw_pixel( -x + x0,  y + y0 );
        GUT_draw_pixel( -y + x0,  x + y0 );
        GUT_draw_pixel( -x + x0, -y + y0 );
        GUT_draw_pixel( -y + x0, -x + y0 );
        GUT_draw_pixel(  x + x0, -y + y0 );
        GUT_draw_pixel(  y + x0, -x + y0 );
        y++;
        if (rad_err < 0)
            rad_err += 2 * y + 1;
        else
            x--, rad_err += 2 * (y - x + 1);
    }
}