// LuaPaint
// Krisztian Pinter 2014

#include <stdio.h>

#include "lua.h"
#include "lualib.h"
#include "lauxlib.h"

#include "GUT.h"

// Segdfggvny egszek Lua tblbl val kiolvassra
// L - a Lua llapot
// keys - a Lua tblbl kiolvasand mezk nevei
// vals - tmb, ahova a fggvny a kiolvasott egszeket rhatja
void load_ints_by_key(lua_State *L, char *keys[], int *vals);

// Begyazott fggvnyek

// Kp inicializlsa, ez az els fggvny amit rajzols eltt meg kell
// hvni a szkriptben
int init_image(lua_State *L);
// Pixel rajzolsa
int draw_pixel(lua_State *L;
// Vonal rajzolsa
int draw_line(lua_State *L);
// Kr rajzolsa
int draw_circle(lua_State *L);

int main(int argc, char *argv[])
{
    if(argc < 3)
    {
        printf("Usage: luapaint <script> <output bmp>");
    }
    else
    {
        lua_State* L = luaL_newstate();
        luaL_openlibs(L);
        // Begyazand fggvnyek regisztrlsa
        lua_register(L, "image" , init_image);
        lua_register(L, "pixel" , draw_pixel);
        lua_register(L, "line"  , draw_line);
        lua_register(L, "circle", draw_circle);
        
        if(luaL_dofile(L, argv[1]))
        {
            int i;
            // Hiba esetn a stack kiratsa
            for(i = -lua_gettop(L); i < 0 ; ++i)
                printf("%s\n", lua_tostring(L, i));
        }
        
        // Lemezre rjuk a kpet
        GUT_output_bmp(argv[2]);
        GUT_close();
        lua_close(L);
    }
    return 0;
}

int init_image(lua_State *L)
{
    GUT_init(lua_tonumber(L, -2), lua_tonumber(L, -1));
    return 0;
}

void load_ints_by_key(lua_State *L, char *keys[], int *vals)
{
    int c = -1;
    while(*keys)
    {
        lua_getfield( L, c, *keys);
        vals[0] = lua_tointeger(L, -1);        
        c--;
        vals++;
        keys++;
    }
    lua_pop(L, -c);
}

int draw_pixel(lua_State *L)
{
    if(!GUT_is_init()) {
        fprintf(stderr,"ERROR: image not initialized!\n");
        exit(1);
    }
    int p[2];
    char *xy_keys[] = {"x","y",0};
    load_ints_by_key( L, xy_keys,  p  );
    GUT_draw_pixel( p[0], p[1] );    
    return 0;
}

int draw_line(lua_State *L)
{
    if(!GUT_is_init()) {
        fprintf(stderr,"ERROR: image not initialized!\n");
        exit(1);
    }
    int col[3];
    int p_2[2];
    int p_1[2];
    
    char *rgb_keys[] = {"r","g","b",0};
    char *xy_keys[] = {"x","y",0};
    
    load_ints_by_key( L, rgb_keys, col );
    load_ints_by_key( L, xy_keys,  p_2 );
    load_ints_by_key( L, xy_keys,  p_1 );
    
    GUT_set_color( col[0], col[1], col[2] );
    GUT_draw_line( p_1[0], p_1[1], p_2[0], p_2[1] );
    
    return 0;
}

int draw_circle(lua_State *L)
{
    if(!GUT_is_init()) {
        fprintf(stderr,"ERROR: image not initialized!\n");
        exit(1);
    }
    int col[3];
    int p[2];
    
    char *rgb_keys[] = {"r","g","b",0};
    char *xy_keys[] = {"x","y",0};
    
    load_ints_by_key( L, rgb_keys, col );
    
    int r = lua_tointeger(L, -1);
    lua_pop(L, 1);
    
    load_ints_by_key( L, xy_keys, p );
    
    GUT_set_color( col[0], col[1], col[2] );
    GUT_draw_circle( p[0], p[1], r );
    
    return 0;
}